@@ -21,6 +21,8 @@
 
 #include "nm-default.h"
 
+#include "agent.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -31,7 +33,6 @@
 #include "utils.h"
 #include "nm-secret-agent-simple.h"
 #include "polkit-agent.h"
-#include "agent.h"
 
 static void
 usage (void)
@@ -125,7 +126,7 @@ secrets_requested (NMSecretAgentSimple *agent,
 	NmCli *nmc = (NmCli *) user_data;
 	gboolean success = FALSE;
 
-	if (nmc->print_output == NMC_PRINT_PRETTY)
+	if (nmc->nmc_config.print_output == NMC_PRINT_PRETTY)
 		nmc_terminal_erase_line ();
 
 	success = get_secrets_from_user (request_id, title, msg, secrets);
@@ -139,6 +140,7 @@ secrets_requested (NMSecretAgentSimple *agent,
 static NMCResultCode
 do_agent_secret (NmCli *nmc, int argc, char **argv)
 {
+	next_arg (nmc, &argc, &argv, NULL);
 	if (nmc->complete)
 		return nmc->return_value;
 
@@ -167,6 +169,7 @@ do_agent_polkit (NmCli *nmc, int argc, char **argv)
 {
 	GError *error = NULL;
 
+	next_arg (nmc, &argc, &argv, NULL);
 	if (nmc->complete)
 		return nmc->return_value;
 
@@ -192,15 +195,22 @@ do_agent_all (NmCli *nmc, int argc, char **argv)
 {
 	NMCResultCode secret_res;
 
+	next_arg (nmc, &argc, &argv, NULL);
 	if (nmc->complete)
 		return nmc->return_value;
 
 	/* Run both secret and polkit agent */
 	secret_res = do_agent_secret (nmc, argc, argv);
-	if (secret_res != NMC_RESULT_SUCCESS)
+	if (secret_res != NMC_RESULT_SUCCESS) {
 		g_printerr ("%s\n", nmc->return_text->str);
+		g_string_truncate (nmc->return_text, 0);
+	}
 
 	nmc->return_value = do_agent_polkit (nmc, argc, argv);
+	if (nmc->return_value != NMC_RESULT_SUCCESS) {
+		g_printerr ("%s\n", nmc->return_text->str);
+		g_string_truncate (nmc->return_text, 0);
+	}
 
 	if (nmc->return_value == NMC_RESULT_SUCCESS && secret_res != NMC_RESULT_SUCCESS)
 		nmc->return_value = secret_res;
@@ -209,25 +219,17 @@ do_agent_all (NmCli *nmc, int argc, char **argv)
 }
 
 static const NMCCommand agent_cmds[] = {
-	{"secret",  do_agent_secret,  usage_agent_secret },
-	{"polkit",  do_agent_polkit,  usage_agent_polkit },
-	{"all",     do_agent_all,     usage_agent_all },
-	{NULL,      do_agent_all,     usage }
+	{ "secret",  do_agent_secret,  usage_agent_secret,  TRUE,   TRUE },
+	{ "polkit",  do_agent_polkit,  usage_agent_polkit,  TRUE,   TRUE },
+	{ "all",     do_agent_all,     usage_agent_all,     TRUE,   TRUE },
+	{ NULL,      do_agent_all,     usage,               TRUE,   TRUE },
 };
 
-
 NMCResultCode
 do_agent (NmCli *nmc, int argc, char **argv)
 {
-	/* Get NMClient object */
-	nmc->get_client (nmc);
+	next_arg (nmc, &argc, &argv, NULL);
+	nmc_do_cmd (nmc, agent_cmds, *argv, argc, argv);
 
-	/* Check whether NetworkManager is running */
-	if (!nm_client_get_nm_running (nmc->client)) {
-		g_string_printf (nmc->return_text, _("Error: NetworkManager is not running."));
-		nmc->return_value = NMC_RESULT_ERROR_NM_NOT_RUNNING;
-		return nmc->return_value;
-	}
-
-	return nmc_do_cmd (nmc, agent_cmds, *argv, argc, argv);
+	return nmc->return_value;
 }
