@@ -19,7 +19,7 @@
 #ifndef __NM_VPN_HELPERS_H__
 #define __NM_VPN_HELPERS_H__
 
-#include <NetworkManager.h>
+#include "NetworkManager.h"
 
 typedef struct {
 	const char *name;
