@@ -41,7 +41,7 @@
  **/
 
 G_DEFINE_TYPE_WITH_CODE (NMSettingMacsec, nm_setting_macsec, NM_TYPE_SETTING,
-                         _nm_register_setting (MACSEC, 1))
+                         _nm_register_setting (MACSEC, NM_SETTING_PRIORITY_HW_BASE))
 NM_SETTING_REGISTER_TYPE (NM_TYPE_SETTING_MACSEC)
 
 #define NM_SETTING_MACSEC_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_SETTING_MACSEC, NMSettingMacsecPrivate))
@@ -251,7 +251,7 @@ verify_macsec_key (const char *key, gboolean cak, GError **error)
 		g_set_error_literal (error,
 		                     NM_CONNECTION_ERROR,
 		                     NM_CONNECTION_ERROR_INVALID_PROPERTY,
-		                     _("the key contais non-hexadecimal characters"));
+		                     _("the key contains non-hexadecimal characters"));
 		return FALSE;
 	}
 
