@@ -18,14 +18,12 @@
  * Copyright 2012 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <string.h>
 
-#include "nm-glib-compat.h"
-
-#include <nm-setting-connection.h>
-#include <nm-setting-olpc-mesh.h>
+#include "nm-setting-connection.h"
+#include "nm-setting-olpc-mesh.h"
 
 #include "nm-device-olpc-mesh.h"
 #include "nm-device-private.h"
@@ -187,7 +185,7 @@ get_setting_type (NMDevice *device)
 	return NM_TYPE_SETTING_OLPC_MESH;
 }
 
-/**************************************************************/
+/*****************************************************************************/
 
 static void
 nm_device_olpc_mesh_init (NMDeviceOlpcMesh *device)
