@@ -108,7 +108,7 @@ nm_remote_connection_error_quark (void)
 	return quark;
 }
 
-/****************************************************************/
+/*****************************************************************************/
 
 static void
 _nm_remote_connection_ensure_inited (NMRemoteConnection *self)
@@ -131,7 +131,7 @@ _nm_remote_connection_ensure_inited (NMRemoteConnection *self)
 	}
 }
 
-/****************************************************************/
+/*****************************************************************************/
 
 static void
 remote_call_dbus_cb (DBusGProxy *proxy, DBusGProxyCall *proxy_call, gpointer user_data)
@@ -213,7 +213,7 @@ proxy_destroy_cb (DBusGProxy* proxy, gpointer user_data) {
 	proxy_set_destroyed (user_data);
 }
 
-/****************************************************************/
+/*****************************************************************************/
 
 static void
 result_cb (RemoteCall *call, DBusGProxyCall *proxy_call, GError *error)
@@ -445,7 +445,7 @@ nm_remote_connection_get_unsaved (NMRemoteConnection *connection)
 	return NM_REMOTE_CONNECTION_GET_PRIVATE (connection)->unsaved;
 }
 
-/****************************************************************/
+/*****************************************************************************/
 
 static void
 updated_get_settings_cb (DBusGProxy *proxy,
@@ -477,7 +477,7 @@ updated_get_settings_cb (DBusGProxy *proxy,
 		priv->visible = FALSE;
 		g_signal_emit (self, signals[VISIBLE], 0, FALSE);
 	} else {
-		gs_unref_object NMConnection *self_alive = NULL;
+		gs_unref_object NMRemoteConnection *self_alive = NULL;
 
 		self_alive = g_object_ref (self);
 		_nm_connection_replace_settings (NM_CONNECTION (self), new_settings);
@@ -531,7 +531,7 @@ properties_changed_cb (DBusGProxy *proxy,
 	}
 }
 
-/****************************************************************/
+/*****************************************************************************/
 
 /**
  * nm_remote_connection_new:
@@ -601,9 +601,10 @@ constructed (GObject *object)
 static gboolean
 init_sync (GInitable *initable, GCancellable *cancellable, GError **error)
 {
-	NMRemoteConnectionPrivate *priv = NM_REMOTE_CONNECTION_GET_PRIVATE (initable);
+	NMRemoteConnection *self = NM_REMOTE_CONNECTION (initable);
+	NMRemoteConnectionPrivate *priv = NM_REMOTE_CONNECTION_GET_PRIVATE (self);
 	GHashTable *hash;
-	gs_unref_object NMConnection *self_alive = NULL;
+	gs_unref_object NMRemoteConnection *self_alive = NULL;
 
 	if (!dbus_g_proxy_call (priv->proxy, "GetSettings", error,
 	                        G_TYPE_INVALID,
@@ -611,9 +612,9 @@ init_sync (GInitable *initable, GCancellable *cancellable, GError **error)
 	                        G_TYPE_INVALID))
 		return FALSE;
 	priv->visible = TRUE;
-	self_alive = g_object_ref (initable);
-	_nm_connection_replace_settings (NM_CONNECTION (initable), hash);
-	g_signal_emit (initable, signals[UPDATED], 0, hash);
+	self_alive = g_object_ref (self);
+	_nm_connection_replace_settings (NM_CONNECTION (self), hash);
+	g_signal_emit (self, signals[UPDATED], 0, hash);
 	g_hash_table_destroy (hash);
 
 	/* Get properties */
@@ -676,7 +677,7 @@ init_get_settings_cb (DBusGProxy *proxy,
 	NMRemoteConnectionPrivate *priv = NM_REMOTE_CONNECTION_GET_PRIVATE (init_data->connection);
 	GHashTable *settings;
 	GError *error = NULL;
-	gs_unref_object NMConnection *self_alive = NULL;
+	gs_unref_object NMRemoteConnection *self_alive = NULL;
 
 	dbus_g_proxy_end_call (proxy, call, &error,
 	                       DBUS_TYPE_G_MAP_OF_MAP_OF_VARIANT, &settings,
@@ -712,6 +713,8 @@ init_async (GAsyncInitable *initable, int io_priority,
 	init_data->connection = NM_REMOTE_CONNECTION (initable);
 	init_data->result = g_simple_async_result_new (G_OBJECT (initable), callback,
 	                                               user_data, init_async);
+	if (cancellable)
+		g_simple_async_result_set_check_cancellable (init_data->result, cancellable);
 
 	dbus_g_proxy_begin_call (priv->proxy, "GetSettings",
 	                         init_get_settings_cb, init_data, NULL,
@@ -796,7 +799,7 @@ set_property (GObject *object, guint prop_id,
 	switch (prop_id) {
 	case PROP_BUS:
 	case PROP_DBUS_CONNECTION:
-		/* Construct only */
+		/* construct-only */
 		/* priv->bus is set from either of two properties so that it (a) remains
 		 * backwards compatible with the previous "bus" property, and that (b)
 		 * it can be created just like an NMObject using the "dbus-connection",
