@@ -28,10 +28,6 @@
 gboolean _nm_ssid_demarshal (GValue *value, GByteArray **dest);
 gboolean _nm_uint_array_demarshal (GValue *value, GArray **dest);
 gboolean _nm_string_array_demarshal (GValue *value, GPtrArray **dest);
-gboolean _nm_object_array_demarshal (GValue *value,
-                                     GPtrArray **dest,
-                                     DBusGConnection *connection,
-                                     NMObjectCreatorFunc func);
 gboolean _nm_ip6_address_array_demarshal (GValue *value, GSList **dest);
 
 #endif /* NM_TYPES_PRIVATE_H */
