@@ -654,6 +654,9 @@ crypto_is_pkcs12_data (const GByteArray *data)
 
 	g_return_val_if_fail (data != NULL, FALSE);
 
+	if (!data->len)
+		return FALSE;
+
 	success = crypto_verify_pkcs12 (data, NULL, &error);
 	if (success == FALSE) {
 		/* If the error was just a decryption error, then it's pkcs#12 */
@@ -705,7 +708,7 @@ crypto_verify_private_key_data (const GByteArray *contents,
 		/* Maybe it's PKCS#8 */
 		tmp = parse_pkcs8_key_file (contents, &is_encrypted, error);
 		if (tmp) {
-			if (crypto_verify_pkcs8 (tmp, is_encrypted, password, error))
+			if (!password || crypto_verify_pkcs8 (tmp, is_encrypted, password, error))
 				format = NM_CRYPTO_FILE_FORMAT_RAW_KEY;
 		} else {
 			g_clear_error (error);
