@@ -37,6 +37,8 @@
 #include "nm-ip6-config.h"
 #include "nm-remote-connection.h"
 
+#include "introspection/org.freedesktop.NetworkManager.Connection.Active.h"
+
 G_DEFINE_TYPE (NMActiveConnection, nm_active_connection, NM_TYPE_OBJECT);
 
 #define NM_ACTIVE_CONNECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_ACTIVE_CONNECTION, NMActiveConnectionPrivate))
@@ -49,6 +51,7 @@ typedef struct {
 	char *specific_object_path;
 	GPtrArray *devices;
 	NMActiveConnectionState state;
+	guint state_flags;
 	gboolean is_default;
 	NMIPConfig *ip4_config;
 	NMDhcpConfig *dhcp4_config;
@@ -57,6 +60,7 @@ typedef struct {
 	NMDhcpConfig *dhcp6_config;
 	gboolean is_vpn;
 	NMDevice *master;
+	NMActiveConnectionStateReason reason;
 } NMActiveConnectionPrivate;
 
 enum {
@@ -68,6 +72,7 @@ enum {
 	PROP_SPECIFIC_OBJECT_PATH,
 	PROP_DEVICES,
 	PROP_STATE,
+	PROP_STATE_FLAGS,
 	PROP_DEFAULT,
 	PROP_IP4_CONFIG,
 	PROP_DHCP4_CONFIG,
@@ -80,6 +85,14 @@ enum {
 	LAST_PROP
 };
 
+enum {
+	STATE_CHANGED,
+
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 /**
  * nm_active_connection_get_connection:
  * @connection: a #NMActiveConnection
@@ -204,6 +217,42 @@ nm_active_connection_get_state (NMActiveConnection *connection)
 	return NM_ACTIVE_CONNECTION_GET_PRIVATE (connection)->state;
 }
 
+/**
+ * nm_active_connection_get_state_flags:
+ * @connection: a #NMActiveConnection
+ *
+ * Gets the active connection's state flags.
+ *
+ * Returns: the state flags
+ *
+ * Since: 1.10
+ **/
+NMActivationStateFlags
+nm_active_connection_get_state_flags (NMActiveConnection *connection)
+{
+	g_return_val_if_fail (NM_IS_ACTIVE_CONNECTION (connection), NM_ACTIVATION_STATE_FLAG_NONE);
+
+	return NM_ACTIVE_CONNECTION_GET_PRIVATE (connection)->state_flags;
+}
+
+/**
+ * nm_active_connection_get_state_reason:
+ * @connection: a #NMActiveConnection
+ *
+ * Gets the reason for active connection's state.
+ *
+ * Returns: the reason
+ *
+ * Since: 1.8
+ **/
+NMActiveConnectionStateReason
+nm_active_connection_get_state_reason (NMActiveConnection *connection)
+{
+	g_return_val_if_fail (NM_IS_ACTIVE_CONNECTION (connection), NM_ACTIVE_CONNECTION_STATE_REASON_UNKNOWN);
+
+	return NM_ACTIVE_CONNECTION_GET_PRIVATE (connection)->reason;
+}
+
 /**
  * nm_active_connection_get_default:
  * @connection: a #NMActiveConnection
@@ -350,10 +399,39 @@ nm_active_connection_init (NMActiveConnection *connection)
 	priv->devices = g_ptr_array_new ();
 }
 
+static void
+state_changed_proxy (NMDBusActiveConnectionProxy *proxy,
+                     NMActiveConnectionState state,
+                     NMActiveConnectionStateReason reason,
+                     gpointer user_data)
+{
+	NMActiveConnection *connection = NM_ACTIVE_CONNECTION (user_data);
+	NMActiveConnectionPrivate *priv = NM_ACTIVE_CONNECTION_GET_PRIVATE (connection);
+
+	priv->state = state;
+	priv->reason = reason;
+	g_signal_emit (connection, signals[STATE_CHANGED], 0, state, reason);
+	g_object_notify (G_OBJECT (connection), NM_ACTIVE_CONNECTION_STATE);
+}
+
+static void
+constructed (GObject *object)
+{
+	GDBusProxy *proxy;
+
+	proxy = _nm_object_get_proxy (NM_OBJECT (object), NM_DBUS_INTERFACE_ACTIVE_CONNECTION);
+	g_signal_connect (proxy, "state-changed",
+	                  G_CALLBACK (state_changed_proxy), object);
+	g_object_unref (proxy);
+
+	G_OBJECT_CLASS (nm_active_connection_parent_class)->constructed (object);
+}
+
 static void
 dispose (GObject *object)
 {
 	NMActiveConnectionPrivate *priv = NM_ACTIVE_CONNECTION_GET_PRIVATE (object);
+	GDBusProxy *proxy;
 
 	g_clear_pointer (&priv->devices, g_ptr_array_unref);
 
@@ -364,6 +442,12 @@ dispose (GObject *object)
 	g_clear_object (&priv->ip6_config);
 	g_clear_object (&priv->dhcp6_config);
 
+	proxy = _nm_object_get_proxy (NM_OBJECT (object), NM_DBUS_INTERFACE_ACTIVE_CONNECTION);
+	if (proxy) {
+		g_signal_handlers_disconnect_by_data (proxy, object);
+		g_object_unref (proxy);
+	}
+
 	G_OBJECT_CLASS (nm_active_connection_parent_class)->dispose (object);
 }
 
@@ -410,6 +494,9 @@ get_property (GObject *object,
 	case PROP_STATE:
 		g_value_set_enum (value, nm_active_connection_get_state (self));
 		break;
+	case PROP_STATE_FLAGS:
+		g_value_set_uint (value, nm_active_connection_get_state_flags (self));
+		break;
 	case PROP_DEFAULT:
 		g_value_set_boolean (value, nm_active_connection_get_default (self));
 		break;
@@ -473,6 +560,7 @@ init_dbus (NMObject *object)
 		{ "specific-object",                         &priv->specific_object_path, demarshal_specific_object_path },
 		{ NM_ACTIVE_CONNECTION_DEVICES,              &priv->devices, NULL, NM_TYPE_DEVICE },
 		{ NM_ACTIVE_CONNECTION_STATE,                &priv->state },
+		{ NM_ACTIVE_CONNECTION_STATE_FLAGS,          &priv->state_flags },
 		{ NM_ACTIVE_CONNECTION_DEFAULT,              &priv->is_default },
 		{ NM_ACTIVE_CONNECTION_IP4_CONFIG,           &priv->ip4_config, NULL, NM_TYPE_IP4_CONFIG },
 		{ NM_ACTIVE_CONNECTION_DHCP4_CONFIG,         &priv->dhcp4_config, NULL, NM_TYPE_DHCP4_CONFIG },
@@ -503,6 +591,7 @@ nm_active_connection_class_init (NMActiveConnectionClass *ap_class)
 
 	/* virtual methods */
 	object_class->get_property = get_property;
+	object_class->constructed = constructed;
 	object_class->dispose = dispose;
 	object_class->finalize = finalize;
 
@@ -598,6 +687,21 @@ nm_active_connection_class_init (NMActiveConnectionClass *ap_class)
 		                    G_PARAM_READABLE |
 		                    G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * NMActiveConnection:state-flags:
+	 *
+	 * The state flags of the active connection.
+	 *
+	 * Since: 1.10
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_STATE_FLAGS,
+		 g_param_spec_uint (NM_ACTIVE_CONNECTION_STATE_FLAGS, "", "",
+		                    0, G_MAXUINT32,
+		                    NM_ACTIVATION_STATE_FLAG_NONE,
+		                    G_PARAM_READABLE |
+		                    G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * NMActiveConnection:default:
 	 *
@@ -693,4 +797,13 @@ nm_active_connection_class_init (NMActiveConnectionClass *ap_class)
 		                      NM_TYPE_DEVICE,
 		                      G_PARAM_READABLE |
 		                      G_PARAM_STATIC_STRINGS));
+
+	/* signals */
+	signals[STATE_CHANGED] =
+		g_signal_new ("state-changed",
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_FIRST,
+		              0, NULL, NULL, NULL,
+		              G_TYPE_NONE, 2,
+		              G_TYPE_UINT, G_TYPE_UINT);
 }
