@@ -27,7 +27,6 @@
 #include "nm-utils.h"
 
 #include "nm-device-team.h"
-#include "nm-device-private.h"
 #include "nm-object-private.h"
 #include "nm-core-internal.h"
 
@@ -39,6 +38,7 @@ typedef struct {
 	char *hw_address;
 	gboolean carrier;
 	GPtrArray *slaves;
+	char *config;
 } NMDeviceTeamPrivate;
 
 enum {
@@ -46,6 +46,7 @@ enum {
 	PROP_HW_ADDRESS,
 	PROP_CARRIER,
 	PROP_SLAVES,
+	PROP_CONFIG,
 
 	LAST_PROP
 };
@@ -64,7 +65,7 @@ nm_device_team_get_hw_address (NMDeviceTeam *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_TEAM (device), NULL);
 
-	return NM_DEVICE_TEAM_GET_PRIVATE (device)->hw_address;
+	return nm_str_not_empty (NM_DEVICE_TEAM_GET_PRIVATE (device)->hw_address);
 }
 
 /**
@@ -101,6 +102,25 @@ nm_device_team_get_slaves (NMDeviceTeam *device)
 	return NM_DEVICE_TEAM_GET_PRIVATE (device)->slaves;
 }
 
+/**
+ * nm_device_team_get_config:
+ * @device: a #NMDeviceTeam
+ *
+ * Gets the current JSON configuration of the #NMDeviceTeam
+ *
+ * Returns: the current configuration. This is the internal string used by the
+ * device, and must not be modified.
+ *
+ * Since: 1.4
+ **/
+const char *
+nm_device_team_get_config (NMDeviceTeam *device)
+{
+	g_return_val_if_fail (NM_IS_DEVICE_TEAM (device), NULL);
+
+	return nm_str_not_empty (NM_DEVICE_TEAM_GET_PRIVATE (device)->config);
+}
+
 static const char *
 get_hw_address (NMDevice *device)
 {
@@ -130,15 +150,13 @@ get_setting_type (NMDevice *device)
 	return NM_TYPE_SETTING_TEAM;
 }
 
-/***********************************************************/
+/*****************************************************************************/
 
 static void
 nm_device_team_init (NMDeviceTeam *device)
 {
 	NMDeviceTeamPrivate *priv = NM_DEVICE_TEAM_GET_PRIVATE (device);
 
-	_nm_device_set_device_type (NM_DEVICE (device), NM_DEVICE_TYPE_TEAM);
-
 	priv->slaves = g_ptr_array_new ();
 }
 
@@ -150,6 +168,7 @@ init_dbus (NMObject *object)
 		{ NM_DEVICE_TEAM_HW_ADDRESS, &priv->hw_address },
 		{ NM_DEVICE_TEAM_CARRIER,    &priv->carrier },
 		{ NM_DEVICE_TEAM_SLAVES,     &priv->slaves, NULL, NM_TYPE_DEVICE },
+		{ NM_DEVICE_TEAM_CONFIG,     &priv->config },
 		{ NULL },
 	};
 
@@ -176,6 +195,7 @@ finalize (GObject *object)
 	NMDeviceTeamPrivate *priv = NM_DEVICE_TEAM_GET_PRIVATE (object);
 
 	g_free (priv->hw_address);
+	g_free (priv->config);
 
 	G_OBJECT_CLASS (nm_device_team_parent_class)->finalize (object);
 }
@@ -198,6 +218,9 @@ get_property (GObject *object,
 	case PROP_SLAVES:
 		g_value_take_boxed (value, _nm_utils_copy_object_array (nm_device_team_get_slaves (device)));
 		break;
+	case PROP_CONFIG:
+		g_value_set_string (value, nm_device_team_get_config (device));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -213,8 +236,6 @@ nm_device_team_class_init (NMDeviceTeamClass *team_class)
 
 	g_type_class_add_private (team_class, sizeof (NMDeviceTeamPrivate));
 
-	_nm_object_class_add_interface (nm_object_class, NM_DBUS_INTERFACE_DEVICE_TEAM);
-
 	/* virtual methods */
 	object_class->dispose = dispose;
 	object_class->finalize = finalize;
@@ -265,4 +286,18 @@ nm_device_team_class_init (NMDeviceTeamClass *team_class)
 		                     G_TYPE_PTR_ARRAY,
 		                     G_PARAM_READABLE |
 		                     G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * NMDeviceTeam:config:
+	 *
+	 * The current JSON configuration of the device.
+	 *
+	 * Since: 1.4
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_CONFIG,
+		 g_param_spec_string (NM_DEVICE_TEAM_CONFIG, "", "",
+		                      NULL,
+		                      G_PARAM_READABLE |
+		                      G_PARAM_STATIC_STRINGS));
 }
