@@ -30,9 +30,7 @@
 #include "nm-device-wimax.h"
 #include "nm-wimax-nsp.h"
 #include "nm-object-private.h"
-#include "nm-object-cache.h"
 #include "nm-core-internal.h"
-#include "nm-device-private.h"
 
 G_DEFINE_TYPE (NMDeviceWimax, nm_device_wimax, NM_TYPE_DEVICE)
 
@@ -91,7 +89,7 @@ nm_device_wimax_get_hw_address (NMDeviceWimax *wimax)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_WIMAX (wimax), NULL);
 
-	return NM_DEVICE_WIMAX_GET_PRIVATE (wimax)->hw_address;
+	return nm_str_not_empty (NM_DEVICE_WIMAX_GET_PRIVATE (wimax)->hw_address);
 }
 
 /**
@@ -300,7 +298,7 @@ nm_device_wimax_get_bsid (NMDeviceWimax *self)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_WIMAX (self), NULL);
 
-	return NM_DEVICE_WIMAX_GET_PRIVATE (self)->bsid;
+	return nm_str_not_empty (NM_DEVICE_WIMAX_GET_PRIVATE (self)->bsid);
 }
 
 static gboolean
@@ -350,13 +348,11 @@ get_hw_address (NMDevice *device)
 	return nm_device_wimax_get_hw_address (NM_DEVICE_WIMAX (device));
 }
 
-/**************************************************************/
+/*****************************************************************************/
 
 static void
 nm_device_wimax_init (NMDeviceWimax *device)
 {
-	_nm_device_set_device_type (NM_DEVICE (device), NM_DEVICE_TYPE_WIMAX);
-
 	g_signal_connect (device,
 	                  "notify::" NM_DEVICE_STATE,
 	                  G_CALLBACK (state_changed_cb),
@@ -531,8 +527,6 @@ nm_device_wimax_class_init (NMDeviceWimaxClass *wimax_class)
 
 	g_type_class_add_private (wimax_class, sizeof (NMDeviceWimaxPrivate));
 
-	_nm_object_class_add_interface (nm_object_class, NM_DBUS_INTERFACE_DEVICE_WIMAX);
-
 	/* virtual methods */
 	object_class->get_property = get_property;
 	object_class->dispose = dispose;
