@@ -683,6 +683,8 @@ nm_secret_agent_old_register_async (NMSecretAgentOld *self,
 
 	simple = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
 	                                    nm_secret_agent_old_register_async);
+	if (cancellable)
+		g_simple_async_result_set_check_cancellable (simple, cancellable);
 
 	if (!check_nm_running (self, &error)) {
 		g_simple_async_result_take_error (simple, error);
@@ -826,6 +828,8 @@ nm_secret_agent_old_unregister_async (NMSecretAgentOld *self,
 
 	simple = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
 	                                    nm_secret_agent_old_unregister_async);
+	if (cancellable)
+		g_simple_async_result_set_check_cancellable (simple, cancellable);
 
 	if (!check_nm_running (self, &error)) {
 		g_simple_async_result_take_error (simple, error);
@@ -1118,7 +1122,7 @@ init_async_got_proxy (GObject *object, GAsyncResult *result, gpointer user_data)
 
 	if (priv->auto_register) {
 		nm_secret_agent_old_register_async (init_data->self, init_data->cancellable,
-		                                init_async_registered, init_data);
+		                                    init_async_registered, init_data);
 	} else
 		init_async_complete (init_data, NULL);
 }
@@ -1139,7 +1143,7 @@ init_async_got_bus (GObject *initable, GAsyncResult *result, gpointer user_data)
 	nmdbus_agent_manager_proxy_new (priv->bus,
 	                                  G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES
 	                                | G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
-	                                NM_DBUS_INTERFACE_AGENT_MANAGER,
+	                                NM_DBUS_SERVICE,
 	                                NM_DBUS_PATH_AGENT_MANAGER,
 	                                init_data->cancellable,
 	                                init_async_got_proxy, init_data);
@@ -1159,6 +1163,8 @@ init_async (GAsyncInitable *initable, int io_priority,
 
 	init_data->simple = g_simple_async_result_new (G_OBJECT (initable), callback,
 	                                               user_data, init_async);
+	if (cancellable)
+		g_simple_async_result_set_check_cancellable (init_data->simple, cancellable);
 
 	_nm_dbus_new_connection_async (cancellable, init_async_got_bus, init_data);
 }
