@@ -21,7 +21,7 @@
 #ifndef __NM_SECRET_AGENT_OLD_H__
 #define __NM_SECRET_AGENT_OLD_H__
 
-#include <nm-types.h>
+#include "nm-types.h"
 
 G_BEGIN_DECLS
 
