@@ -217,7 +217,7 @@ nm_vpn_service_plugin_disconnect (NMVpnServicePlugin *plugin, GError **err)
 		break;
 	case NM_VPN_SERVICE_STATE_STARTING:
 		_emit_failure (plugin, NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
-		/* fallthru */
+		/* fall through */
 	case NM_VPN_SERVICE_STATE_STARTED:
 		nm_vpn_service_plugin_set_state (plugin, NM_VPN_SERVICE_STATE_STOPPING);
 		ret = NM_VPN_SERVICE_PLUGIN_GET_CLASS (plugin)->disconnect (plugin, err);
@@ -996,11 +996,6 @@ init_sync (GInitable *initable, GCancellable *cancellable, GError **error)
 		goto out;
 
 	priv->dbus_vpn_service_plugin = nmdbus_vpn_plugin_skeleton_new ();
-	if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (priv->dbus_vpn_service_plugin),
-	                                       connection,
-	                                       NM_VPN_DBUS_PLUGIN_PATH,
-	                                       error))
-		goto out;
 
 	_nm_dbus_bind_properties (plugin, priv->dbus_vpn_service_plugin);
 	_nm_dbus_bind_methods (plugin, priv->dbus_vpn_service_plugin,
@@ -1015,6 +1010,12 @@ init_sync (GInitable *initable, GCancellable *cancellable, GError **error)
 	                       "SetFailure", impl_vpn_service_plugin_set_failure,
 	                       NULL);
 
+	if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (priv->dbus_vpn_service_plugin),
+	                                       connection,
+	                                       NM_VPN_DBUS_PLUGIN_PATH,
+	                                       error))
+		goto out;
+
 	nm_vpn_service_plugin_set_connection (plugin, connection);
 	nm_vpn_service_plugin_set_state (plugin, NM_VPN_SERVICE_STATE_INIT);
 
@@ -1047,11 +1048,11 @@ set_property (GObject *object, guint prop_id,
 
 	switch (prop_id) {
 	case PROP_DBUS_SERVICE_NAME:
-		/* Construct-only */
+		/* construct-only */
 		priv->dbus_service_name = g_value_dup_string (value);
 		break;
 	case PROP_DBUS_WATCH_PEER:
-		/* Construct-only */
+		/* construct-only */
 		priv->dbus_watch_peer = g_value_get_boolean (value);
 		break;
 	case PROP_STATE:
@@ -1109,6 +1110,11 @@ dispose (GObject *object)
 		g_error_free (err);
 	}
 
+	if (priv->dbus_vpn_service_plugin) {
+		g_dbus_interface_skeleton_unexport (G_DBUS_INTERFACE_SKELETON (priv->dbus_vpn_service_plugin));
+		g_clear_object (&priv->dbus_vpn_service_plugin);
+	}
+
 	G_OBJECT_CLASS (nm_vpn_service_plugin_parent_class)->dispose (object);
 }
 
