@@ -24,6 +24,7 @@
 
 #include <string.h>
 
+#include "nm-utils/nm-hash-utils.h"
 #include "nm-setting-connection.h"
 #include "nm-auth-subject.h"
 #include "nm-auth-manager.h"
@@ -131,7 +132,7 @@ nm_auth_chain_new_subject (NMAuthSubject *subject,
 
 	self = g_slice_new0 (NMAuthChain);
 	self->refcount = 1;
-	self->data = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, chain_data_free);
+	self->data = g_hash_table_new_full (nm_str_hash, g_str_equal, g_free, chain_data_free);
 	self->done_func = done_func;
 	self->user_data = user_data;
 	self->context = context ? g_object_ref (context) : NULL;
@@ -301,12 +302,10 @@ auth_call_cancel (gpointer user_data)
 {
 	AuthCall *call = user_data;
 
-	if (call->cancellable) {
+	if (nm_clear_g_cancellable (&call->cancellable)) {
 		/* we don't free call immediately. Instead we cancel the async operation
 		 * and set cancellable to NULL. pk_call_cb() will check for this and
 		 * do the final cleanup. */
-		g_cancellable_cancel (call->cancellable);
-		g_clear_object (&call->cancellable);
 	} else {
 		g_source_remove (call->call_idle_id);
 		auth_call_free (call);
@@ -319,7 +318,7 @@ pk_call_cb (GObject *object, GAsyncResult *result, gpointer user_data)
 {
 	AuthCall *call = user_data;
 	GError *error = NULL;
-	gboolean is_authorized, is_challenge;
+	gboolean is_authorized = FALSE, is_challenge = FALSE;
 	guint call_result = NM_AUTH_CALL_RESULT_UNKNOWN;
 
 	nm_auth_manager_polkit_authority_check_authorization_finish (NM_AUTH_MANAGER (object),
