@@ -77,8 +77,7 @@ typedef struct {
 	GArray *methods;
 } NMExportedObjectClassInfo;
 
-GQuark nm_exported_object_class_info_quark (void);
-G_DEFINE_QUARK (NMExportedObjectClassInfo, nm_exported_object_class_info)
+static NM_CACHED_QUARK_FCN ("NMExportedObjectClassInfo", nm_exported_object_class_info_quark)
 
 /*****************************************************************************/
 
@@ -273,7 +272,7 @@ nm_exported_object_class_add_interface (NMExportedObjectClass *object_class,
 		classinfo = g_slice_new (NMExportedObjectClassInfo);
 		classinfo->skeleton_types = NULL;
 		classinfo->methods = g_array_new (FALSE, FALSE, sizeof (NMExportedObjectDBusMethodImpl));
-		classinfo->properties = g_hash_table_new (g_str_hash, g_str_equal);
+		classinfo->properties = g_hash_table_new (nm_str_hash, g_str_equal);
 		g_type_set_qdata (G_TYPE_FROM_CLASS (object_class),
 		                  nm_exported_object_class_info_quark (), classinfo);
 	}
@@ -388,8 +387,7 @@ nm_exported_object_meta_marshal (GClosure *closure, GValue *return_value,
 	g_free (local_param_values);
 }
 
-GQuark _skeleton_data_quark (void);
-G_DEFINE_QUARK (skeleton-data, _skeleton_data);
+static NM_CACHED_QUARK_FCN ("skeleton-data", _skeleton_data_quark)
 
 typedef struct {
 	GBinding **prop_bindings;
@@ -581,7 +579,7 @@ _create_export_path (NMExportedObjectClass *klass)
 	p = strchr (class_export_path, '%');
 	if (p) {
 		if (G_UNLIKELY (!prefix_counters))
-			prefix_counters = g_hash_table_new (g_str_hash, g_str_equal);
+			prefix_counters = g_hash_table_new (nm_str_hash, g_str_equal);
 
 		nm_assert (p[1] == 'l');
 		nm_assert (p[2] == 'l');
@@ -595,7 +593,7 @@ _create_export_path (NMExportedObjectClass *klass)
 		}
 
 		NM_PRAGMA_WARNING_DISABLE("-Wformat-nonliteral")
-		return g_strdup_printf (class_export_path, (long long unsigned) (++(*counter)));
+		return g_strdup_printf (class_export_path, (unsigned long long) (++(*counter)));
 		NM_PRAGMA_WARNING_REENABLE
 	}
 
