@@ -31,20 +31,18 @@
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <signal.h>
-
-/* Cannot include <net/if.h> due to conflict with <linux/if.h>.
- * Forward declare if_nametoindex. */
-extern unsigned int if_nametoindex (const char *__ifname);
+#include <linux/rtnetlink.h>
 
 #include "main-utils.h"
 #include "NetworkManagerUtils.h"
-#include "nm-linux-platform.h"
-#include "nm-dhcp-manager.h"
-#include "nm-rdisc.h"
-#include "nm-lndp-rdisc.h"
+#include "platform/nm-linux-platform.h"
+#include "platform/nm-platform-utils.h"
+#include "dhcp/nm-dhcp-manager.h"
+#include "ndisc/nm-ndisc.h"
+#include "ndisc/nm-lndp-ndisc.h"
 #include "nm-utils.h"
 #include "nm-setting-ip6-config.h"
-#include "nm-sd.h"
+#include "systemd/nm-sd.h"
 
 #if !defined(NM_DIST_VERSION)
 # define NM_DIST_VERSION VERSION
@@ -52,8 +50,14 @@ extern unsigned int if_nametoindex (const char *__ifname);
 
 #define NMIH_PID_FILE_FMT NMRUNDIR "/nm-iface-helper-%d.pid"
 
-static GMainLoop *main_loop = NULL;
-static int ifindex = -1;
+/*****************************************************************************/
+
+static struct {
+	GMainLoop *main_loop;
+	int ifindex;
+} gl/*obal*/ = {
+	.ifindex = -1,
+};
 
 static struct {
 	gboolean slaac;
@@ -66,6 +70,7 @@ static struct {
 	int tempaddr;
 	char *ifname;
 	char *uuid;
+	char *stable_id;
 	char *dhcp4_address;
 	char *dhcp4_clientid;
 	char *dhcp4_hostname;
@@ -83,6 +88,16 @@ static struct {
 	.priority_v6 = NM_PLATFORM_ROUTE_METRIC_DEFAULT_IP6,
 };
 
+/*****************************************************************************/
+
+#define _NMLOG_PREFIX_NAME      "nm-iface-helper"
+#define _NMLOG(level, domain, ...) \
+    nm_log ((level), (domain), global_opt.ifname, NULL, \
+            "iface-helper: " _NM_UTILS_MACRO_FIRST (__VA_ARGS__) \
+            _NM_UTILS_MACRO_REST (__VA_ARGS__))
+
+/*****************************************************************************/
+
 static void
 dhcp4_state_changed (NMDhcpClient *client,
                      NMDhcpState state,
@@ -93,35 +108,50 @@ dhcp4_state_changed (NMDhcpClient *client,
 {
 	static NMIP4Config *last_config = NULL;
 	NMIP4Config *existing;
+	gs_unref_ptrarray GPtrArray *ip4_dev_route_blacklist = NULL;
 
 	g_return_if_fail (!ip4_config || NM_IS_IP4_CONFIG (ip4_config));
 
-	nm_log_dbg (LOGD_DHCP4, "(%s): new DHCPv4 client state %d", global_opt.ifname, state);
+	_LOGD (LOGD_DHCP4, "new DHCPv4 client state %d", state);
 
 	switch (state) {
 	case NM_DHCP_STATE_BOUND:
 		g_assert (ip4_config);
-		existing = nm_ip4_config_capture (ifindex, FALSE);
-		if (last_config)
-			nm_ip4_config_subtract (existing, last_config);
+		g_assert (nm_ip4_config_get_ifindex (ip4_config) == gl.ifindex);
 
-		nm_ip4_config_merge (existing, ip4_config, NM_IP_CONFIG_MERGE_DEFAULT);
-		if (!nm_ip4_config_commit (existing, ifindex, TRUE, global_opt.priority_v4))
-			nm_log_warn (LOGD_DHCP4, "(%s): failed to apply DHCPv4 config", global_opt.ifname);
+		existing = nm_ip4_config_capture (nm_platform_get_multi_idx (NM_PLATFORM_GET),
+		                                  NM_PLATFORM_GET, gl.ifindex, FALSE);
+		if (last_config)
+			nm_ip4_config_subtract (existing, last_config, 0);
+
+		nm_ip4_config_merge (existing, ip4_config, NM_IP_CONFIG_MERGE_DEFAULT, 0);
+		nm_ip4_config_add_dependent_routes (existing,
+		                                    RT_TABLE_MAIN,
+		                                    global_opt.priority_v4,
+		                                    &ip4_dev_route_blacklist);
+		if (!nm_ip4_config_commit (existing,
+		                           NM_PLATFORM_GET,
+		                           NM_IP_ROUTE_TABLE_SYNC_MODE_MAIN))
+			_LOGW (LOGD_DHCP4, "failed to apply DHCPv4 config");
+
+		nm_platform_ip4_dev_route_blacklist_set (NM_PLATFORM_GET,
+		                                         gl.ifindex,
+		                                         ip4_dev_route_blacklist);
 
 		if (last_config)
 			g_object_unref (last_config);
-		last_config = nm_ip4_config_new (nm_dhcp_client_get_ifindex (client));
+		last_config = nm_ip4_config_new (nm_platform_get_multi_idx (NM_PLATFORM_GET),
+		                                 nm_dhcp_client_get_ifindex (client));
 		nm_ip4_config_replace (last_config, ip4_config, NULL);
 		break;
 	case NM_DHCP_STATE_TIMEOUT:
 	case NM_DHCP_STATE_DONE:
 	case NM_DHCP_STATE_FAIL:
 		if (global_opt.dhcp4_required) {
-			nm_log_warn (LOGD_DHCP4, "(%s): DHCPv4 timed out or failed, quitting...", global_opt.ifname);
-			g_main_loop_quit (main_loop);
+			_LOGW (LOGD_DHCP4, "DHCPv4 timed out or failed, quitting...");
+			g_main_loop_quit (gl.main_loop);
 		} else
-			nm_log_warn (LOGD_DHCP4, "(%s): DHCPv4 timed out or failed", global_opt.ifname);
+			_LOGW (LOGD_DHCP4, "DHCPv4 timed out or failed");
 		break;
 	default:
 		break;
@@ -129,137 +159,100 @@ dhcp4_state_changed (NMDhcpClient *client,
 }
 
 static void
-rdisc_config_changed (NMRDisc *rdisc, NMRDiscConfigMap changed, gpointer user_data)
+ndisc_config_changed (NMNDisc *ndisc, const NMNDiscData *rdata, guint changed_int, gpointer user_data)
 {
-	static NMIP6Config *rdisc_config = NULL;
+	NMNDiscConfigMap changed = changed_int;
+	static NMIP6Config *ndisc_config = NULL;
 	NMIP6Config *existing;
-	static int system_support = -1;
-	guint32 ifa_flags = 0x00;
-	int i;
 
-	if (system_support == -1) {
-		/*
-		 * Check, whether kernel is recent enough, to help user space handling RA.
+	existing = nm_ip6_config_capture (nm_platform_get_multi_idx (NM_PLATFORM_GET),
+	                                  NM_PLATFORM_GET, gl.ifindex, FALSE, global_opt.tempaddr);
+	if (ndisc_config)
+		nm_ip6_config_subtract (existing, ndisc_config, 0);
+	else {
+		ndisc_config = nm_ip6_config_new (nm_platform_get_multi_idx (NM_PLATFORM_GET),
+		                                  gl.ifindex);
+	}
+
+	if (changed & NM_NDISC_CONFIG_ADDRESSES) {
+		guint8 plen;
+		guint32 ifa_flags;
+
+		/* Check, whether kernel is recent enough to help user space handling RA.
 		 * If it's not supported, we have no ipv6-privacy and must add autoconf
-		 * addresses as /128.
-		 * The reason for the /128 is to prevent the kernel
-		 * from adding a prefix route for this address.
-		 **/
-		system_support = nm_platform_check_support_kernel_extended_ifa_flags (NM_PLATFORM_GET);
-	}
-
-	if (system_support)
-		ifa_flags = IFA_F_NOPREFIXROUTE;
-	if (global_opt.tempaddr == NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_TEMP_ADDR
-	    || global_opt.tempaddr == NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_PUBLIC_ADDR)
-	{
-		/* without system_support, this flag will be ignored. Still set it, doesn't seem to do any harm. */
-		ifa_flags |= IFA_F_MANAGETEMPADDR;
-	}
-
-	existing = nm_ip6_config_capture (ifindex, FALSE, global_opt.tempaddr);
-	if (rdisc_config)
-		nm_ip6_config_subtract (existing, rdisc_config);
-	else
-		rdisc_config = nm_ip6_config_new (ifindex);
-
-	if (changed & NM_RDISC_CONFIG_GATEWAYS) {
-		/* Use the first gateway as ordered in router discovery cache. */
-		if (rdisc->gateways->len) {
-			NMRDiscGateway *gateway = &g_array_index (rdisc->gateways, NMRDiscGateway, 0);
-
-			nm_ip6_config_set_gateway (rdisc_config, &gateway->address);
+		 * addresses as /128. The reason for the /128 is to prevent the kernel
+		 * from adding a prefix route for this address. */
+		ifa_flags = 0;
+		if (nm_platform_check_kernel_support (NM_PLATFORM_GET,
+		                                      NM_PLATFORM_KERNEL_SUPPORT_EXTENDED_IFA_FLAGS)) {
+			ifa_flags |= IFA_F_NOPREFIXROUTE;
+			if (NM_IN_SET (global_opt.tempaddr, NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_TEMP_ADDR,
+			                                    NM_SETTING_IP6_CONFIG_PRIVACY_PREFER_PUBLIC_ADDR))
+				ifa_flags |= IFA_F_MANAGETEMPADDR;
+			plen = 64;
 		} else
-			nm_ip6_config_set_gateway (rdisc_config, NULL);
+			plen = 128;
+
+		nm_ip6_config_reset_addresses_ndisc (ndisc_config,
+		                                     rdata->addresses,
+		                                     rdata->addresses_n,
+		                                     plen,
+		                                     ifa_flags);
 	}
 
-	if (changed & NM_RDISC_CONFIG_ADDRESSES) {
-		/* Rebuild address list from router discovery cache. */
-		nm_ip6_config_reset_addresses (rdisc_config);
-
-		/* rdisc->addresses contains at most max_addresses entries.
-		 * This is different from what the kernel does, which
-		 * also counts static and temporary addresses when checking
-		 * max_addresses.
-		 **/
-		for (i = 0; i < rdisc->addresses->len; i++) {
-			NMRDiscAddress *discovered_address = &g_array_index (rdisc->addresses, NMRDiscAddress, i);
-			NMPlatformIP6Address address;
-
-			memset (&address, 0, sizeof (address));
-			address.address = discovered_address->address;
-			address.plen = system_support ? 64 : 128;
-			address.timestamp = discovered_address->timestamp;
-			address.lifetime = discovered_address->lifetime;
-			address.preferred = discovered_address->preferred;
-			if (address.preferred > address.lifetime)
-				address.preferred = address.lifetime;
-			address.source = NM_IP_CONFIG_SOURCE_RDISC;
-			address.n_ifa_flags = ifa_flags;
-
-			nm_ip6_config_add_address (rdisc_config, &address);
-		}
+	if (NM_FLAGS_ANY (changed,  NM_NDISC_CONFIG_ROUTES
+	                          | NM_NDISC_CONFIG_GATEWAYS)) {
+		nm_ip6_config_reset_routes_ndisc (ndisc_config,
+		                                  rdata->gateways,
+		                                  rdata->gateways_n,
+		                                  rdata->routes,
+		                                  rdata->routes_n,
+		                                  RT_TABLE_MAIN,
+		                                  global_opt.priority_v6,
+		                                  nm_platform_check_kernel_support (NM_PLATFORM_GET,
+		                                                                    NM_PLATFORM_KERNEL_SUPPORT_RTA_PREF));
 	}
 
-	if (changed & NM_RDISC_CONFIG_ROUTES) {
-		/* Rebuild route list from router discovery cache. */
-		nm_ip6_config_reset_routes (rdisc_config);
-
-		for (i = 0; i < rdisc->routes->len; i++) {
-			NMRDiscRoute *discovered_route = &g_array_index (rdisc->routes, NMRDiscRoute, i);
-			NMPlatformIP6Route route;
-
-			/* Only accept non-default routes.  The router has no idea what the
-			 * local configuration or user preferences are, so sending routes
-			 * with a prefix length of 0 is quite rude and thus ignored.
-			 */
-			if (   discovered_route->plen > 0
-			    && discovered_route->plen <= 128) {
-				memset (&route, 0, sizeof (route));
-				route.network = discovered_route->network;
-				route.plen = discovered_route->plen;
-				route.gateway = discovered_route->gateway;
-				route.source = NM_IP_CONFIG_SOURCE_RDISC;
-				route.metric = global_opt.priority_v6;
-
-				nm_ip6_config_add_route (rdisc_config, &route);
-			}
-		}
-	}
-
-	if (changed & NM_RDISC_CONFIG_DHCP_LEVEL) {
+	if (changed & NM_NDISC_CONFIG_DHCP_LEVEL) {
 		/* Unsupported until systemd DHCPv6 is ready */
 	}
 
-	if (changed & NM_RDISC_CONFIG_HOP_LIMIT)
-		nm_platform_sysctl_set_ip6_hop_limit_safe (NM_PLATFORM_GET, global_opt.ifname, rdisc->hop_limit);
+	if (changed & NM_NDISC_CONFIG_HOP_LIMIT)
+		nm_platform_sysctl_set_ip6_hop_limit_safe (NM_PLATFORM_GET, global_opt.ifname, rdata->hop_limit);
 
-	if (changed & NM_RDISC_CONFIG_MTU) {
+	if (changed & NM_NDISC_CONFIG_MTU) {
 		char val[16];
+		char sysctl_path_buf[NM_UTILS_SYSCTL_IP_CONF_PATH_BUFSIZE];
 
-		g_snprintf (val, sizeof (val), "%d", rdisc->mtu);
-		nm_platform_sysctl_set (NM_PLATFORM_GET, nm_utils_ip6_property_path (global_opt.ifname, "mtu"), val);
+		g_snprintf (val, sizeof (val), "%d", rdata->mtu);
+		nm_platform_sysctl_set (NM_PLATFORM_GET, NMP_SYSCTL_PATHID_ABSOLUTE (nm_utils_sysctl_ip_conf_path (AF_INET6, sysctl_path_buf, global_opt.ifname, "mtu")), val);
 	}
 
-	nm_ip6_config_merge (existing, rdisc_config, NM_IP_CONFIG_MERGE_DEFAULT);
-	if (!nm_ip6_config_commit (existing, ifindex, TRUE))
-		nm_log_warn (LOGD_IP6, "(%s): failed to apply IPv6 config", global_opt.ifname);
+	nm_ip6_config_merge (existing, ndisc_config, NM_IP_CONFIG_MERGE_DEFAULT, 0);
+	nm_ip6_config_add_dependent_routes (existing,
+	                                    RT_TABLE_MAIN,
+	                                    global_opt.priority_v6);
+	if (!nm_ip6_config_commit (existing,
+	                           NM_PLATFORM_GET,
+	                           NM_IP_ROUTE_TABLE_SYNC_MODE_MAIN,
+	                           NULL))
+		_LOGW (LOGD_IP6, "failed to apply IPv6 config");
 }
 
 static void
-rdisc_ra_timeout (NMRDisc *rdisc, gpointer user_data)
+ndisc_ra_timeout (NMNDisc *ndisc, gpointer user_data)
 {
 	if (global_opt.slaac_required) {
-		nm_log_warn (LOGD_IP6, "(%s): IPv6 timed out or failed, quitting...", global_opt.ifname);
-		g_main_loop_quit (main_loop);
+		_LOGW (LOGD_IP6, "IPv6 timed out or failed, quitting...");
+		g_main_loop_quit (gl.main_loop);
 	} else
-		nm_log_warn (LOGD_IP6, "(%s): IPv6 timed out or failed", global_opt.ifname);
+		_LOGW (LOGD_IP6, "IPv6 timed out or failed");
 }
 
 static gboolean
 quit_handler (gpointer user_data)
 {
-	g_main_loop_quit (main_loop);
+	g_main_loop_quit (gl.main_loop);
 	return G_SOURCE_REMOVE;
 }
 
@@ -271,15 +264,16 @@ setup_signals (void)
 	g_unix_signal_add (SIGTERM, quit_handler, NULL);
 }
 
-static void
+static gboolean
 do_early_setup (int *argc, char **argv[])
 {
 	gint64 priority64_v4 = -1;
 	gint64 priority64_v6 = -1;
 	GOptionEntry options[] = {
 		/* Interface/IP config */
-		{ "ifname", 'i', 0, G_OPTION_ARG_STRING, &global_opt.ifname, N_("The interface to manage"), N_("eth0") },
-		{ "uuid", 'u', 0, G_OPTION_ARG_STRING, &global_opt.uuid, N_("Connection UUID"), N_("661e8cd0-b618-46b8-9dc9-31a52baaa16b") },
+		{ "ifname", 'i', 0, G_OPTION_ARG_STRING, &global_opt.ifname, N_("The interface to manage"), "eth0" },
+		{ "uuid", 'u', 0, G_OPTION_ARG_STRING, &global_opt.uuid, N_("Connection UUID"),  "661e8cd0-b618-46b8-9dc9-31a52baaa16b" },
+		{ "stable-id", '\0', 0, G_OPTION_ARG_STRING, &global_opt.stable_id, N_("Connection Token for Stable IDs"),  "eth" },
 		{ "slaac", 's', 0, G_OPTION_ARG_NONE, &global_opt.slaac, N_("Whether to manage IPv6 SLAAC"), NULL },
 		{ "slaac-required", '6', 0, G_OPTION_ARG_NONE, &global_opt.slaac_required, N_("Whether SLAAC must be successful"), NULL },
 		{ "slaac-tempaddr", 't', 0, G_OPTION_ARG_INT, &global_opt.tempaddr, N_("Use an IPv6 temporary privacy address"), NULL },
@@ -313,25 +307,28 @@ do_early_setup (int *argc, char **argv[])
 	                                NULL,
 	                                NULL,
 	                                _("nm-iface-helper is a small, standalone process that manages a single network interface.")))
-		exit (1);
+		return FALSE;
 
 	if (priority64_v4 >= 0 && priority64_v4 <= G_MAXUINT32)
 		global_opt.priority_v4 = (guint32) priority64_v4;
 	if (priority64_v6 >= 0 && priority64_v6 <= G_MAXUINT32)
 		global_opt.priority_v6 = (guint32) priority64_v6;
+	return TRUE;
 }
 
 static void
 ip6_address_changed (NMPlatform *platform,
-                     NMPObjectType obj_type,
+                     int obj_type_i,
                      int iface,
                      NMPlatformIP6Address *addr,
-                     NMPlatformSignalChangeType change_type,
-                     NMRDisc *rdisc)
+                     int change_type_i,
+                     NMNDisc *ndisc)
 {
+	const NMPlatformSignalChangeType change_type = change_type_i;
+
 	if (   (change_type == NM_PLATFORM_SIGNAL_CHANGED && addr->n_ifa_flags & IFA_F_DADFAILED)
 	    || (change_type == NM_PLATFORM_SIGNAL_REMOVED && addr->n_ifa_flags & IFA_F_TENTATIVE))
-		nm_rdisc_dad_failed (rdisc, &addr->address);
+		nm_ndisc_dad_failed (ndisc, &addr->address);
 }
 
 int
@@ -342,18 +339,26 @@ main (int argc, char *argv[])
 	gboolean wrote_pidfile = FALSE;
 	gs_free char *pidfile = NULL;
 	gs_unref_object NMDhcpClient *dhcp4_client = NULL;
-	gs_unref_object NMRDisc *rdisc = NULL;
+	gs_unref_object NMNDisc *ndisc = NULL;
 	GByteArray *hwaddr = NULL;
 	size_t hwaddr_len = 0;
 	gconstpointer tmp;
 	gs_free NMUtilsIPv6IfaceId *iid = NULL;
 	guint sd_id;
+	char sysctl_path_buf[NM_UTILS_SYSCTL_IP_CONF_PATH_BUFSIZE];
 
 	nm_g_type_init ();
 
 	setpgid (getpid (), getpid ());
 
-	do_early_setup (&argc, &argv);
+	if (!do_early_setup (&argc, &argv))
+		return 1;
+
+	nm_logging_set_syslog_identifier ("nm-iface-helper");
+	nm_logging_set_prefix ("%s[%ld] (%s): ",
+	                       _NMLOG_PREFIX_NAME,
+	                       (long) getpid (),
+	                       global_opt.ifname ?: "???");
 
 	if (global_opt.g_fatal_warnings) {
 		GLogLevelFlags fatal_mask;
@@ -365,22 +370,22 @@ main (int argc, char *argv[])
 
 	if (global_opt.show_version) {
 		fprintf (stdout, NM_DIST_VERSION "\n");
-		exit (0);
+		return 0;
 	}
 
 	nm_main_utils_ensure_root ();
 
 	if (!global_opt.ifname || !global_opt.uuid) {
 		fprintf (stderr, _("An interface name and UUID are required\n"));
-		exit (1);
+		return 1;
 	}
 
-	ifindex = if_nametoindex (global_opt.ifname);
-	if (ifindex <= 0) {
+	gl.ifindex = nmp_utils_if_nametoindex (global_opt.ifname);
+	if (gl.ifindex <= 0) {
 		fprintf (stderr, _("Failed to find interface index for %s (%s)\n"), global_opt.ifname, strerror (errno));
-		exit (1);
+		return 1;
 	}
-	pidfile = g_strdup_printf (NMIH_PID_FILE_FMT, ifindex);
+	pidfile = g_strdup_printf (NMIH_PID_FILE_FMT, gl.ifindex);
 	nm_main_utils_ensure_not_running_pidfile (pidfile);
 
 	nm_main_utils_ensure_rundir ();
@@ -392,7 +397,7 @@ main (int argc, char *argv[])
 		fprintf (stderr,
 		         _("%s.  Please use --help to see a list of valid options.\n"),
 		         error->message);
-		exit (1);
+		return 1;
 	} else if (bad_domains) {
 		fprintf (stderr,
 		         _("Ignoring unrecognized log domain(s) '%s' passed on command line.\n"),
@@ -408,26 +413,25 @@ main (int argc, char *argv[])
 			fprintf (stderr, _("Could not daemonize: %s [error %u]\n"),
 			         g_strerror (saved_errno),
 			         saved_errno);
-			exit (1);
+			return 1;
 		}
 		if (nm_main_utils_write_pidfile (pidfile))
 			wrote_pidfile = TRUE;
 	}
 
 	/* Set up unix signal handling - before creating threads, but after daemonizing! */
-	main_loop = g_main_loop_new (NULL, FALSE);
+	gl.main_loop = g_main_loop_new (NULL, FALSE);
 	setup_signals ();
 
-	nm_logging_syslog_openlog (global_opt.logging_backend
-	                           ? global_opt.logging_backend
-	                           : (global_opt.debug ? "debug" : NULL));
+	nm_logging_syslog_openlog (global_opt.logging_backend,
+	                           global_opt.debug);
 
-	nm_log_info (LOGD_CORE, "nm-iface-helper (version " NM_DIST_VERSION ") is starting...");
+	_LOGI (LOGD_CORE, "nm-iface-helper (version " NM_DIST_VERSION ") is starting...");
 
 	/* Set up platform interaction layer */
 	nm_linux_platform_setup ();
 
-	tmp = nm_platform_link_get_address (NM_PLATFORM_GET, ifindex, &hwaddr_len);
+	tmp = nm_platform_link_get_address (NM_PLATFORM_GET, gl.ifindex, &hwaddr_len);
 	if (tmp) {
 		hwaddr = g_byte_array_sized_new (hwaddr_len);
 		g_byte_array_append (hwaddr, tmp, hwaddr_len);
@@ -440,25 +444,27 @@ main (int argc, char *argv[])
 		bytes = nm_utils_hexstr2bin (global_opt.iid_str);
 		if (!bytes || g_bytes_get_size (bytes) != sizeof (*iid)) {
 			fprintf (stderr, _("(%s): Invalid IID %s\n"), global_opt.ifname, global_opt.iid_str);
-			exit (1);
+			return 1;
 		}
 		iid = g_bytes_unref_to_data (bytes, &ignored);
 	}
 
 	if (global_opt.dhcp4_address) {
-		nm_platform_sysctl_set (NM_PLATFORM_GET, nm_utils_ip4_property_path (global_opt.ifname, "promote_secondaries"), "1");
+		nm_platform_sysctl_set (NM_PLATFORM_GET, NMP_SYSCTL_PATHID_ABSOLUTE (nm_utils_sysctl_ip_conf_path (AF_INET, sysctl_path_buf, global_opt.ifname, "promote_secondaries")), "1");
 
 		dhcp4_client = nm_dhcp_manager_start_ip4 (nm_dhcp_manager_get (),
+		                                          nm_platform_get_multi_idx (NM_PLATFORM_GET),
 		                                          global_opt.ifname,
-		                                          ifindex,
+		                                          gl.ifindex,
 		                                          hwaddr,
 		                                          global_opt.uuid,
+		                                          RT_TABLE_MAIN,
 		                                          global_opt.priority_v4,
 		                                          !!global_opt.dhcp4_hostname,
 		                                          global_opt.dhcp4_hostname,
 		                                          global_opt.dhcp4_fqdn,
 		                                          global_opt.dhcp4_clientid,
-		                                          45,
+		                                          NM_DHCP_TIMEOUT_DEFAULT,
 		                                          NULL,
 		                                          global_opt.dhcp4_address);
 		g_assert (dhcp4_client);
@@ -469,98 +475,125 @@ main (int argc, char *argv[])
 	}
 
 	if (global_opt.slaac) {
-		nm_platform_link_set_user_ipv6ll_enabled (NM_PLATFORM_GET, ifindex, TRUE);
+		NMUtilsStableType stable_type = NM_UTILS_STABLE_TYPE_UUID;
+		const char *stable_id = global_opt.uuid;
 
-		rdisc = nm_lndp_rdisc_new (NM_PLATFORM_GET, ifindex, global_opt.ifname, global_opt.uuid, global_opt.addr_gen_mode, NULL);
-		g_assert (rdisc);
+		nm_platform_link_set_user_ipv6ll_enabled (NM_PLATFORM_GET, gl.ifindex, TRUE);
+
+		if (   global_opt.stable_id
+		    && (global_opt.stable_id[0] >= '0' && global_opt.stable_id[0] <= '9')
+		    && global_opt.stable_id[1] == ' ') {
+			/* strict parsing of --stable-id, which is the numeric stable-type
+			 * and the ID, joined with one space. For now, only support stable-types
+			 * from 0 to 9. */
+			stable_type = (global_opt.stable_id[0] - '0');
+			stable_id = &global_opt.stable_id[2];
+		}
+		ndisc = nm_lndp_ndisc_new (NM_PLATFORM_GET, gl.ifindex, global_opt.ifname,
+		                           stable_type, stable_id,
+		                           global_opt.addr_gen_mode,
+		                           NM_NDISC_NODE_TYPE_HOST,
+		                           NULL);
+		g_assert (ndisc);
 
 		if (iid)
-			nm_rdisc_set_iid (rdisc, *iid);
+			nm_ndisc_set_iid (ndisc, *iid);
 
-		nm_platform_sysctl_set (NM_PLATFORM_GET, nm_utils_ip6_property_path (global_opt.ifname, "accept_ra"), "1");
-		nm_platform_sysctl_set (NM_PLATFORM_GET, nm_utils_ip6_property_path (global_opt.ifname, "accept_ra_defrtr"), "0");
-		nm_platform_sysctl_set (NM_PLATFORM_GET, nm_utils_ip6_property_path (global_opt.ifname, "accept_ra_pinfo"), "0");
-		nm_platform_sysctl_set (NM_PLATFORM_GET, nm_utils_ip6_property_path (global_opt.ifname, "accept_ra_rtr_pref"), "0");
+		nm_platform_sysctl_set (NM_PLATFORM_GET, NMP_SYSCTL_PATHID_ABSOLUTE (nm_utils_sysctl_ip_conf_path (AF_INET6, sysctl_path_buf, global_opt.ifname, "accept_ra")), "1");
+		nm_platform_sysctl_set (NM_PLATFORM_GET, NMP_SYSCTL_PATHID_ABSOLUTE (nm_utils_sysctl_ip_conf_path (AF_INET6, sysctl_path_buf, global_opt.ifname, "accept_ra_defrtr")), "0");
+		nm_platform_sysctl_set (NM_PLATFORM_GET, NMP_SYSCTL_PATHID_ABSOLUTE (nm_utils_sysctl_ip_conf_path (AF_INET6, sysctl_path_buf, global_opt.ifname, "accept_ra_pinfo")), "0");
+		nm_platform_sysctl_set (NM_PLATFORM_GET, NMP_SYSCTL_PATHID_ABSOLUTE (nm_utils_sysctl_ip_conf_path (AF_INET6, sysctl_path_buf, global_opt.ifname, "accept_ra_rtr_pref")), "0");
 
 		g_signal_connect (NM_PLATFORM_GET,
 		                  NM_PLATFORM_SIGNAL_IP6_ADDRESS_CHANGED,
 		                  G_CALLBACK (ip6_address_changed),
-		                  rdisc);
-		g_signal_connect (rdisc,
-		                  NM_RDISC_CONFIG_CHANGED,
-		                  G_CALLBACK (rdisc_config_changed),
+		                  ndisc);
+		g_signal_connect (ndisc,
+		                  NM_NDISC_CONFIG_RECEIVED,
+		                  G_CALLBACK (ndisc_config_changed),
 		                  NULL);
-		g_signal_connect (rdisc,
-		                  NM_RDISC_RA_TIMEOUT,
-		                  G_CALLBACK (rdisc_ra_timeout),
+		g_signal_connect (ndisc,
+		                  NM_NDISC_RA_TIMEOUT,
+		                  G_CALLBACK (ndisc_ra_timeout),
 		                  NULL);
-		nm_rdisc_start (rdisc);
+		nm_ndisc_start (ndisc);
 	}
 
 	sd_id = nm_sd_event_attach_default ();
 
-	g_main_loop_run (main_loop);
+	g_main_loop_run (gl.main_loop);
 
 	g_clear_pointer (&hwaddr, g_byte_array_unref);
 
 	if (pidfile && wrote_pidfile)
 		unlink (pidfile);
 
-	nm_log_info (LOGD_CORE, "exiting");
+	_LOGI (LOGD_CORE, "exiting");
 
 	nm_clear_g_source (&sd_id);
-	exit (0);
+	g_clear_pointer (&gl.main_loop, g_main_loop_unref);
+	return 0;
 }
 
-/*******************************************************/
+/*****************************************************************************/
+
+const NMDhcpClientFactory *const _nm_dhcp_manager_factories[4] = {
+	&_nm_dhcp_client_factory_internal,
+};
+
+/*****************************************************************************/
 /* Stub functions */
 
+#include "nm-config.h"
+#include "devices/nm-device.h"
+#include "nm-active-connection.h"
+#include "nm-bus-manager.h"
+
 void
 nm_main_config_reload (int signal)
 {
-	nm_log_info (LOGD_CORE, "reloading configuration not supported");
+	_LOGI (LOGD_CORE, "reloading configuration not supported");
 }
 
-gconstpointer nm_config_get (void);
-const char *nm_config_get_dhcp_client (gpointer unused);
-gboolean nm_config_get_configure_and_quit (gpointer unused);
-gconstpointer nm_bus_manager_get (void);
-void nm_bus_manager_register_object (gpointer unused, gpointer object);
-void nm_bus_manager_unregister_object (gpointer unused, gpointer object);
-GType nm_device_get_type (void);
-GType nm_active_connection_get_type (void);
-
-gconstpointer
+NMConfig *
 nm_config_get (void)
 {
 	return GUINT_TO_POINTER (1);
 }
 
-const char *
-nm_config_get_dhcp_client (gpointer unused)
+NMConfigData *
+nm_config_get_data_orig (NMConfig *config)
 {
-	return "internal";
+	return GUINT_TO_POINTER (1);
+}
+
+char *
+nm_config_data_get_value (const NMConfigData *config_data, const char *group, const char *key, NMConfigGetValueFlags flags)
+{
+	return NULL;
 }
 
 gboolean
-nm_config_get_configure_and_quit (gpointer unused)
+nm_config_get_configure_and_quit (NMConfig *config)
 {
 	return TRUE;
 }
 
-gconstpointer
+NMBusManager *
 nm_bus_manager_get (void)
 {
 	return GUINT_TO_POINTER (1);
 }
 
 void
-nm_bus_manager_register_object (gpointer unused, gpointer object)
+nm_bus_manager_register_object (NMBusManager *bus_manager,
+                                GDBusObjectSkeleton *object)
 {
 }
 
 void
-nm_bus_manager_unregister_object (gpointer unused, gpointer object)
+nm_bus_manager_unregister_object (NMBusManager *bus_manager,
+                                  GDBusObjectSkeleton *object)
 {
 }
 
