@@ -22,6 +22,8 @@
 #ifndef __NETWORKMANAGER_LOGGING_H__
 #define __NETWORKMANAGER_LOGGING_H__
 
+#include "nm-core-types.h"
+
 #ifdef __NM_TEST_UTILS_H__
 #error nm-test-utils.h must be included as last header
 #endif
@@ -65,18 +67,20 @@ typedef enum  { /*< skip >*/
 	LOGD_DISPATCH   = (1LL << 33),
 	LOGD_AUDIT      = (1LL << 34),
 	LOGD_SYSTEMD    = (1LL << 35),
+	LOGD_VPN_PLUGIN = (1LL << 36),
+	LOGD_PROXY      = (1LL << 37),
 
 	__LOGD_MAX,
-	LOGD_ALL       = ((__LOGD_MAX - 1LL) << 1) - 1LL,
+	LOGD_ALL       = (((__LOGD_MAX - 1LL) << 1) - 1LL),
 	LOGD_DEFAULT   = LOGD_ALL & ~(
 	                              LOGD_DBUS_PROPS |
 	                              LOGD_WIFI_SCAN |
+	                              LOGD_VPN_PLUGIN |
 	                              0),
 
 	/* aliases: */
 	LOGD_DHCP       = LOGD_DHCP4 | LOGD_DHCP6,
 	LOGD_IP         = LOGD_IP4 | LOGD_IP6,
-	LOGD_HW         = LOGD_PLATFORM,
 } NMLogDomain;
 
 /* Log levels */
@@ -95,57 +99,100 @@ typedef enum  { /*< skip >*/
 	_LOGL_N, /* the number of logging levels including "OFF" */
 } NMLogLevel;
 
-#define nm_log_err(domain, ...)     nm_log (LOGL_ERR,   (domain), __VA_ARGS__)
-#define nm_log_warn(domain, ...)    nm_log (LOGL_WARN,  (domain), __VA_ARGS__)
-#define nm_log_info(domain, ...)    nm_log (LOGL_INFO,  (domain), __VA_ARGS__)
-#define nm_log_dbg(domain, ...)     nm_log (LOGL_DEBUG, (domain), __VA_ARGS__)
-#define nm_log_trace(domain, ...)   nm_log (LOGL_TRACE, (domain), __VA_ARGS__)
+#define nm_log_err(domain, ...)     nm_log (LOGL_ERR,   (domain),  NULL, NULL, __VA_ARGS__)
+#define nm_log_warn(domain, ...)    nm_log (LOGL_WARN,  (domain),  NULL, NULL, __VA_ARGS__)
+#define nm_log_info(domain, ...)    nm_log (LOGL_INFO,  (domain),  NULL, NULL, __VA_ARGS__)
+#define nm_log_dbg(domain, ...)     nm_log (LOGL_DEBUG, (domain),  NULL, NULL, __VA_ARGS__)
+#define nm_log_trace(domain, ...)   nm_log (LOGL_TRACE, (domain),  NULL, NULL, __VA_ARGS__)
+
+//#define _NM_LOG_FUNC G_STRFUNC
+#define _NM_LOG_FUNC NULL
 
 /* A wrapper for the _nm_log_impl() function that adds call site information.
  * Contrary to nm_log(), it unconditionally calls the function without
  * checking whether logging for the given level and domain is enabled. */
-#define _nm_log(level, domain, error, ...) \
+#define _nm_log(level, domain, error, ifname, con_uuid, ...) \
     G_STMT_START { \
-        _nm_log_impl (__FILE__, __LINE__, G_STRFUNC, (level), (domain), (error), ""__VA_ARGS__); \
+        _nm_log_impl (__FILE__, __LINE__, \
+                      _NM_LOG_FUNC, \
+                      (level), \
+                      (domain), \
+                      (error), \
+                      (ifname), \
+                      (con_uuid), \
+                      ""__VA_ARGS__); \
     } G_STMT_END
 
 /* nm_log() only evaluates it's argument list after checking
  * whether logging for the given level/domain is enabled.  */
-#define nm_log(level, domain, ...) \
+#define nm_log(level, domain, ifname, con_uuid, ...) \
     G_STMT_START { \
         if (nm_logging_enabled ((level), (domain))) { \
-            _nm_log (level, domain, 0, __VA_ARGS__); \
+            _nm_log (level, domain, 0, ifname, con_uuid, __VA_ARGS__); \
         } \
     } G_STMT_END
 
 
-#define _nm_log_ptr(level, domain, self, prefix, ...) \
-   nm_log ((level), (domain), "%s[%p] " _NM_UTILS_MACRO_FIRST(__VA_ARGS__), (prefix) ?: "", self _NM_UTILS_MACRO_REST(__VA_ARGS__))
+#define _nm_log_ptr(level, domain, ifname, con_uuid, self, prefix, ...) \
+   nm_log ((level), \
+           (domain), \
+           (ifname), \
+           (con_uuid), \
+           "%s[%p] " _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
+           (prefix) ?: "", \
+           self _NM_UTILS_MACRO_REST(__VA_ARGS__))
+
+static inline gboolean
+_nm_log_ptr_is_debug (NMLogLevel level)
+{
+	return level <= LOGL_DEBUG;
+}
 
 /* log a message for an object (with providing a generic @self pointer) */
-#define nm_log_ptr(level, domain, self, prefix, ...) \
+#define nm_log_ptr(level, domain, ifname, con_uuid, self, prefix, ...) \
     G_STMT_START { \
-        NM_PRAGMA_WARNING_DISABLE("-Wtautological-compare") \
-        if ((level) <= LOGL_DEBUG) { \
-            _nm_log_ptr ((level), (domain), (self), (prefix), __VA_ARGS__); \
+        if (_nm_log_ptr_is_debug (level)) { \
+            _nm_log_ptr ((level), \
+                         (domain), \
+                         (ifname), \
+                         (con_uuid), \
+                         (self), \
+                         (prefix), \
+                         __VA_ARGS__); \
         } else { \
             const char *__prefix = (prefix); \
             \
-            nm_log ((level), (domain), "%s%s" _NM_UTILS_MACRO_FIRST(__VA_ARGS__), __prefix ?: "", __prefix ? " " : "" _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
+            nm_log ((level), \
+                    (domain), \
+                    (ifname), \
+                    (con_uuid), \
+                    "%s%s" _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
+                    __prefix ?: "", \
+                    __prefix ? " " : "" _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
         } \
-        NM_PRAGMA_WARNING_REENABLE \
     } G_STMT_END
 
 
-#define _nm_log_obj(level, domain, self, prefix, ...) \
-    _nm_log_ptr ((level), (domain), (self), prefix, __VA_ARGS__)
+#define _nm_log_obj(level, domain, ifname, con_uuid, self, prefix, ...) \
+    _nm_log_ptr ((level), \
+                 (domain), \
+                 (ifname), \
+                 (con_uuid), \
+                 (self), \
+                 prefix, \
+                 __VA_ARGS__)
 
 /* log a message for an object (with providing a @self pointer to a GObject).
  * Contrary to nm_log_ptr(), @self must be a GObject type (or %NULL).
  * As of now, nm_log_obj() is identical to nm_log_ptr(), but we might change that */
-#define nm_log_obj(level, domain, self, prefix, ...) \
-    nm_log_ptr ((level), (domain), (self), prefix, __VA_ARGS__)
-
+#define nm_log_obj(level, domain, ifname, con_uuid, self, prefix, ...) \
+    nm_log_ptr ((level), \
+                (domain), \
+                (ifname), \
+                (con_uuid), \
+                (self), \
+                prefix, \
+                __VA_ARGS__)
 
 void _nm_log_impl (const char *file,
                    guint line,
@@ -153,12 +200,24 @@ void _nm_log_impl (const char *file,
                    NMLogLevel level,
                    NMLogDomain domain,
                    int error,
+                   const char *ifname,
+                   const char *con_uuid,
                    const char *fmt,
-                   ...) __attribute__((__format__ (__printf__, 7, 8)));
+                   ...) _nm_printf (9, 10);
 
 const char *nm_logging_level_to_string (void);
 const char *nm_logging_domains_to_string (void);
-gboolean nm_logging_enabled (NMLogLevel level, NMLogDomain domain);
+
+extern NMLogDomain _nm_logging_enabled_state[_LOGL_N_REAL];
+static inline gboolean
+nm_logging_enabled (NMLogLevel level, NMLogDomain domain)
+{
+	nm_assert (((guint) level) < G_N_ELEMENTS (_nm_logging_enabled_state));
+	return    (((guint) level) < G_N_ELEMENTS (_nm_logging_enabled_state))
+	       && !!(_nm_logging_enabled_state[level] & domain);
+}
+
+NMLogLevel nm_logging_get_level (NMLogDomain domain);
 
 const char *nm_logging_all_levels_to_string (void);
 const char *nm_logging_all_domains_to_string (void);
@@ -167,7 +226,12 @@ gboolean nm_logging_setup (const char  *level,
                            const char  *domains,
                            char       **bad_domains,
                            GError     **error);
-void     nm_logging_syslog_openlog (const char *logging_backend);
+
+void nm_logging_set_syslog_identifier (const char *domain);
+void nm_logging_set_prefix (const char *format, ...) _nm_printf (1, 2);
+
+void     nm_logging_syslog_openlog (const char *logging_backend, gboolean debug);
+gboolean nm_logging_syslog_enabled (void);
 
 /*****************************************************************************/
 
@@ -209,7 +273,7 @@ void     nm_logging_syslog_openlog (const char *logging_backend);
 /*****************************************************************************/
 
 /* Some implementation define a second set of logging macros, for a separate
- * use. As with the _LOGD() macro familiy above, the exact implementation
+ * use. As with the _LOGD() macro family above, the exact implementation
  * depends on the file that uses them.
  * Still, it encourages a common pattern to have the common set of macros
  * like _LOG2D(), _LOG2I(), etc. and have _LOG2t() which by default
@@ -246,8 +310,56 @@ void     nm_logging_syslog_openlog (const char *logging_backend);
 #define _LOG2t_err(errsv, ...) G_STMT_START { if (FALSE) { _NMLOG2_err (errsv, LOGL_TRACE, __VA_ARGS__); } } G_STMT_END
 #endif
 
+#define _NMLOG3_ENABLED(level) ( nm_logging_enabled ((level), (_NMLOG3_DOMAIN)) )
+
+#define _LOG3T(...)          _NMLOG3 (LOGL_TRACE, __VA_ARGS__)
+#define _LOG3D(...)          _NMLOG3 (LOGL_DEBUG, __VA_ARGS__)
+#define _LOG3I(...)          _NMLOG3 (LOGL_INFO , __VA_ARGS__)
+#define _LOG3W(...)          _NMLOG3 (LOGL_WARN , __VA_ARGS__)
+#define _LOG3E(...)          _NMLOG3 (LOGL_ERR  , __VA_ARGS__)
+
+#define _LOG3T_ENABLED(...)  _NMLOG3_ENABLED (LOGL_TRACE, ##__VA_ARGS__)
+#define _LOG3D_ENABLED(...)  _NMLOG3_ENABLED (LOGL_DEBUG, ##__VA_ARGS__)
+#define _LOG3I_ENABLED(...)  _NMLOG3_ENABLED (LOGL_INFO , ##__VA_ARGS__)
+#define _LOG3W_ENABLED(...)  _NMLOG3_ENABLED (LOGL_WARN , ##__VA_ARGS__)
+#define _LOG3E_ENABLED(...)  _NMLOG3_ENABLED (LOGL_ERR  , ##__VA_ARGS__)
+
+#define _LOG3T_err(errsv, ...) _NMLOG3_err (errsv, LOGL_TRACE, __VA_ARGS__)
+#define _LOG3D_err(errsv, ...) _NMLOG3_err (errsv, LOGL_DEBUG, __VA_ARGS__)
+#define _LOG3I_err(errsv, ...) _NMLOG3_err (errsv, LOGL_INFO , __VA_ARGS__)
+#define _LOG3W_err(errsv, ...) _NMLOG3_err (errsv, LOGL_WARN , __VA_ARGS__)
+#define _LOG3E_err(errsv, ...) _NMLOG3_err (errsv, LOGL_ERR  , __VA_ARGS__)
+
+#ifdef NM_MORE_LOGGING
+#define _LOG3t_ENABLED(...)    _NMLOG3_ENABLED (LOGL_TRACE, ##__VA_ARGS__)
+#define _LOG3t(...)            _NMLOG3 (LOGL_TRACE, __VA_ARGS__)
+#define _LOG3t_err(errsv, ...) _NMLOG3_err (errsv, LOGL_TRACE, __VA_ARGS__)
+#else
+/* still call the logging macros to get compile time checks, but they will be optimized out. */
+#define _LOG3t_ENABLED(...)    ( FALSE && (_NMLOG3_ENABLED (LOGL_TRACE, ##__VA_ARGS__)) )
+#define _LOG3t(...)            G_STMT_START { if (FALSE) { _NMLOG3 (LOGL_TRACE, __VA_ARGS__); } } G_STMT_END
+#define _LOG3t_err(errsv, ...) G_STMT_START { if (FALSE) { _NMLOG3_err (errsv, LOGL_TRACE, __VA_ARGS__); } } G_STMT_END
+#endif
+
 extern void (*_nm_logging_clear_platform_logging_cache) (void);
 
 /*****************************************************************************/
 
+#define __NMLOG_DEFAULT(level, domain, prefix, ...) \
+	G_STMT_START { \
+		nm_log ((level), (domain), NULL, NULL, \
+		        "%s: " _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
+		        (prefix) \
+		        _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
+	} G_STMT_END
+
+#define __NMLOG_DEFAULT_WITH_ADDR(level, domain, prefix, ...) \
+	G_STMT_START { \
+		nm_log ((level), (domain), NULL, NULL, \
+		        "%s[%p]: " _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
+		        (prefix), \
+		        (self) \
+		        _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
+	} G_STMT_END
+
 #endif /* __NETWORKMANAGER_LOGGING_H__ */
