@@ -38,13 +38,17 @@
 #define USE_UPOWER                      1
 #define _NMLOG_PREFIX_NAME              "sleep-monitor-up"
 
-#elif defined (SUSPEND_RESUME_SYSTEMD)
+#elif defined (SUSPEND_RESUME_SYSTEMD) || defined (SUSPEND_RESUME_ELOGIND)
 
 #define SUSPEND_DBUS_NAME               "org.freedesktop.login1"
 #define SUSPEND_DBUS_PATH               "/org/freedesktop/login1"
 #define SUSPEND_DBUS_INTERFACE          "org.freedesktop.login1.Manager"
 #define USE_UPOWER                      0
+#if defined (SUSPEND_RESUME_SYSTEMD)
 #define _NMLOG_PREFIX_NAME              "sleep-monitor-sd"
+#else
+#define _NMLOG_PREFIX_NAME              "sleep-monitor-el"
+#endif
 
 #elif defined(SUSPEND_RESUME_CONSOLEKIT)
 
@@ -60,12 +64,21 @@
 
 #else
 
-#error define one of SUSPEND_RESUME_SYSTEMD, SUSPEND_RESUME_CONSOLEKIT, or SUSPEND_RESUME_UPOWER
+#error define one of SUSPEND_RESUME_SYSTEMD, SUSPEND_RESUME_ELOGIND, SUSPEND_RESUME_CONSOLEKIT, or SUSPEND_RESUME_UPOWER
 
 #endif
 
+/*****************************************************************************/
+
+enum {
+	SLEEPING,
+	LAST_SIGNAL,
+};
+
+static guint signals[LAST_SIGNAL] = {0};
+
 struct _NMSleepMonitor {
-	GObject parent_instance;
+	GObject parent;
 
 	GDBusProxy *proxy;
 
@@ -81,29 +94,19 @@ struct _NMSleepMonitor {
 };
 
 struct _NMSleepMonitorClass {
-	GObjectClass parent_class;
+	GObjectClass parent;
 };
 
-enum {
-	SLEEPING,
-	LAST_SIGNAL,
-};
-static guint signals[LAST_SIGNAL] = {0};
-
 G_DEFINE_TYPE (NMSleepMonitor, nm_sleep_monitor, G_TYPE_OBJECT);
 
-static void sleep_signal (NMSleepMonitor *self, gboolean is_about_to_suspend);
+/*****************************************************************************/
+
+#define _NMLOG_DOMAIN      LOGD_SUSPEND
+#define _NMLOG(level, ...) __NMLOG_DEFAULT (level, _NMLOG_DOMAIN, _NMLOG_PREFIX_NAME, __VA_ARGS__)
 
 /*****************************************************************************/
 
-#define _NMLOG_DOMAIN                     LOGD_SUSPEND
-#define _NMLOG(level, ...) \
-    G_STMT_START { \
-        nm_log ((level), _NMLOG_DOMAIN, \
-                "%s: " _NM_UTILS_MACRO_FIRST (__VA_ARGS__), \
-                _NMLOG_PREFIX_NAME \
-                _NM_UTILS_MACRO_REST (__VA_ARGS__)); \
-    } G_STMT_END
+static void sleep_signal (NMSleepMonitor *self, gboolean is_about_to_suspend);
 
 /*****************************************************************************/
 
@@ -131,7 +134,7 @@ drop_inhibitor (NMSleepMonitor *self, gboolean force)
 
 	if (self->inhibit_fd >= 0) {
 		_LOGD ("inhibit: dropping sleep inhibitor %d", self->inhibit_fd);
-		close (self->inhibit_fd);
+		nm_close (self->inhibit_fd);
 		self->inhibit_fd = -1;
 	}
 
@@ -337,6 +340,8 @@ on_proxy_acquired (GObject *object,
 #endif
 }
 
+/*****************************************************************************/
+
 static void
 nm_sleep_monitor_init (NMSleepMonitor *self)
 {
